@@ -170,6 +170,7 @@ class Sql
      */
     private function resultSetContainsUniqueKey($db, $table, array $fields_meta)
     {
+        $columns = $GLOBALS['dbi']->getColumns($db, $table);
         $resultSetColumnNames = array();
         foreach ($fields_meta as $oneMeta) {
             $resultSetColumnNames[] = $oneMeta->name;
@@ -181,6 +182,10 @@ class Sql
                 foreach ($indexColumns as $indexColumnName => $dummy) {
                     if (in_array($indexColumnName, $resultSetColumnNames)) {
                         $numberFound++;
+                    } else if (!in_array($indexColumnName, $columns)) {
+                        $numberFound++;
+                    } else if (strpos($columns[$indexColumnName]['Extra'], 'INVISIBLE') !== false) {
+                        $numberFound++;
                     }
                 }
                 if ($numberFound == count($indexColumns)) {
@@ -1161,6 +1166,11 @@ EOT;
             // "Showing rows..." message
             // $_SESSION['tmpval']['max_rows'] = 'all';
             $unlim_num_rows = $num_rows;
+        } elseif ($this->isAppendLimitClause($analyzed_sql_results) && $_SESSION['tmpval']['max_rows'] > $num_rows) {
+            // When user has not defined a limit in query and total rows in
+            // result are less than max_rows to display, there is no need
+            // to count total rows for that query again
+            $unlim_num_rows = $_SESSION['tmpval']['pos'] + $num_rows;
         } elseif ($analyzed_sql_results['querytype'] == 'SELECT'
             || $analyzed_sql_results['is_subquery']
         ) {
@@ -1455,6 +1465,7 @@ EOT;
      * @param DisplayResults $displayResultsObject DisplayResult instance
      * @param array          $extra_data           extra data
      * @param string         $pmaThemeImage        uri of the theme image
+     * @param array|null     $profiling_results    profiling results
      * @param object         $result               executed query results
      * @param string         $sql_query            sql query
      * @param string         $complete_query       complete sql query
@@ -1463,7 +1474,7 @@ EOT;
      */
     private function getQueryResponseForNoResultsReturned(array $analyzed_sql_results, $db,
         $table, $message_to_show, $num_rows, $displayResultsObject, $extra_data,
-        $pmaThemeImage, $result, $sql_query, $complete_query
+        $pmaThemeImage, $profiling_results, $result, $sql_query, $complete_query
     ) {
         if ($this->isDeleteTransformationInfo($analyzed_sql_results)) {
             $this->deleteTransformationInfo($db, $table, $analyzed_sql_results);
@@ -1522,6 +1533,17 @@ EOT;
                     $analyzed_sql_results, true
                 );
 
+                if (isset($profiling_results)) {
+                    $header   = $response->getHeader();
+                    $scripts  = $header->getScripts();
+                    $scripts->addFile('sql.js');
+                    $html_output .= $this->getHtmlForProfilingChart(
+                        $url_query,
+                        $db,
+                        isset($profiling_results) ? $profiling_results : []
+                    );
+                }
+
                 $html_output .= $displayResultsObject->getCreateViewQueryResultOp(
                     $analyzed_sql_results
                 );
@@ -1706,10 +1728,11 @@ EOT;
             } while ($GLOBALS['dbi']->moreResults() && $GLOBALS['dbi']->nextResult());
 
         } else {
-            if (isset($result) && $result !== false) {
+            $fields_meta = array();
+            if (isset($result) && ! is_bool($result)) {
                 $fields_meta = $GLOBALS['dbi']->getFieldsMeta($result);
-                $fields_cnt  = count($fields_meta);
             }
+            $fields_cnt = count($fields_meta);
             $_SESSION['is_multi_query'] = false;
             $displayResultsObject->setProperties(
                 $unlim_num_rows,
@@ -1731,12 +1754,14 @@ EOT;
                 $browse_dist
             );
 
-            $table_html .= $displayResultsObject->getTable(
-                $result,
-                $displayParts,
-                $analyzed_sql_results,
-                $is_limited_display
-            );
+            if (! is_bool($result)) {
+                $table_html .= $displayResultsObject->getTable(
+                    $result,
+                    $displayParts,
+                    $analyzed_sql_results,
+                    $is_limited_display
+                );
+            }
             $GLOBALS['dbi']->freeResult($result);
         }
 
@@ -2216,6 +2241,10 @@ EOT;
                 isset($extra_data) ? $extra_data : null
             );
 
+        if ($GLOBALS['dbi']->moreResults()) {
+            $GLOBALS['dbi']->nextResult();
+        }
+
         $operations = new Operations();
         $warning_messages = $operations->getWarningMessagesArray();
 
@@ -2227,7 +2256,7 @@ EOT;
                 $analyzed_sql_results, $db, $table,
                 isset($message_to_show) ? $message_to_show : null,
                 $num_rows, $displayResultsObject, $extra_data,
-                $pmaThemeImage, isset($result) ? $result : null,
+                $pmaThemeImage, $profiling_results, isset($result) ? $result : null,
                 $sql_query, isset($complete_query) ? $complete_query : null
             );
         } else {
